# Loading module:
# 1. archive.yml
# 2. recipe.yml
# 3. Load_decadal.R (V)
setwd('/esarchive/scratch/aho/git/auto-s2s/')
library(yaml)
library(startR)
#Does not work now. Why??
#source("https://earth.bsc.es/gitlab/es/csoperational/-/raw/master/R/get_regrid_params.R")
source('/esarchive/scratch/aho/git/csoperational/R/get_regrid_params.R')

  archive <- read_yaml("/esarchive/scratch/aho/git/auto-s2s/conf/archive_decadal.yml")
  recipe <- read_yaml("/esarchive/scratch/aho/git/auto-s2s/modules/data_load/recipe_decadal.yml")
  #-------------------------
  # Read from recipe:
  #-------------------------
  model <- recipe$Analysis$Datasets$System$name #'HadGEM3'
  reference <- recipe$Analysis$Datasets$Reference$name #'era5'
  member <- strsplit(recipe$Analysis$Datasets$System$member, ',')[[1]] #c("r1i1p1f2", "r2i1p1f2")
  var <- recipe$Analysis$Variables$name #'tas'
  freq <- recipe$Analysis$Variables$freq #monthly_mean
  lat_min <- as.numeric(recipe$Analysis$Region$latmin) #0
  lat_max <- as.numeric(recipe$Analysis$Region$latmax) #10
  lon_min <- as.numeric(recipe$Analysis$Region$lonmin) #0
  lon_max <- as.numeric(recipe$Analysis$Region$lonmax) #10
  sdates <- as.numeric(recipe$Analysis$Time$sdate$start):as.numeric(recipe$Analysis$Time$sdate$end) #1960:2015
#  fyear <- as.numeric(recipe$Analysis$Time$fyear$start):as.numeric(recipe$Analysis$Time$fyear$end) #1:1
  time <- as.numeric(recipe$Analysis$Time$leadtimemin):as.numeric(recipe$Analysis$Time$leadtimemax)
  regrid <- recipe$Analysis$Regrid$type # to_system

  # only used for daily data
  season <- recipe$Analysis$Time$season

  #-------------------------
  # Read from archive:
  #-------------------------
  table <- archive$archive$System[[model]][[freq]]$table[[var]]  #'Amon'
  grid <- archive$archive$System[[model]][[freq]]$grid[[var]]
  version <- archive$archive$System[[model]][[freq]]$version[[var]]
  if (identical(member, 'all')) {
    member <- strsplit(archive$archive$System[[model]]$member, ',')[[1]]
  }

  #-------------------------
  # derived from above:
  #-------------------------
  if (lon_min >= 0) {
    circularsort <- CircularSort(0, 360)
  } else {
    circularsort <- CircularSort(-180, 180)
  }
  # generate transform params for system and ref
  regrid_params <- get_regrid_params(recipe, archive$archive)

  # Only if the time length in each chunk may differ that we need largest_dims_length to be TRUE. Otherwise, set FALSE to increase efficiency.
  need_largest_dims_length <- ifelse(model == 'EC-Earth3-i2', TRUE, FALSE) 

  #time for daily data. If recipe asks for time step, we don't need to calculate here
#  fdays <- get_days_dcpp(fyears = fyears, season = season, init = 'Nov', calendar = '360-days')

  
#-------------------------------------------
# Step 1: Load the exp
#-------------------------------------------
  #monthly and daily
  path <- file.path(archive$archive$src, archive$archive$System[[model]]$src, 
                    '$ensemble$', table, '$var$', grid, version)
  files <- paste0('$var$_', table, '_*_*_s$syear$-$ensemble$_', grid, '_$chunk$.nc')

#monthly
  hcst <- Start(dat = file.path(path, files),
                var = var,
                ensemble = member,
                syear = paste0(sdates),
                chunk = 'all',
                chunk_depends = 'syear',
                time = indices(time),
                time_across = 'chunk',
                merge_across_dims = TRUE,
                largest_dims_length = need_largest_dims_length,
                latitude = values(list(lat_min, lat_max)),
                latitude_reorder = Sort(decreasing = TRUE),
                longitude = values(list(lon_min, lon_max)),
                longitude_reorder = circularsort,
                transform = regrid_params$fcst.transform,
                transform_extra_cells = 2,
                transform_params = list(grid = regrid_params$fcst.gridtype, #nc file
                                        method = regrid_params$fcst.gridmethod),
                transform_vars = c('latitude', 'longitude'),
                synonims = list(longitude = c('lon', 'longitude'),
                                latitude = c('lat', 'latitude')),
                return_vars = list(latitude = NULL, longitude = NULL, 
                                   time = c('syear', 'chunk')),
                retrieve = T)


#daily (not done yet)
  data <- Start(dat = path,
                var = variable,
                syear = paste0(sdates),
                aux = 'all',
                aux_depends = 'syear',
                latitude = values(list(lat_min, lat_max)),
                latitude_reorder = Sort(decreasing = TRUE),
                longitude = values(list(lon_min, lon_max)),
                longitude_reorder = CircularSort(0, 360),
                fday = indices(fdays),
                ensemble = members,
                synonims = list(fday = c('fday', 'time'),
                               longitude = c('lon', 'longitude'),
                               latitude = c('lat', 'latitude')),
                return_vars = list(latitude = NULL, longitude = NULL,
                                   fday = c('syear', 'aux')),
                retrieve = F)



#-------------------------------------------
# Step 2. Load the reference
#-------------------------------------------
  path.obs <- file.path(archive$archive$src, archive$archive$Reference[[reference]]$src, freq,
                        paste0(var, archive$archive$Reference[[reference]][[freq]][[var]]))
  files.obs <- paste0('$var$_$file_date$.nc')


  dates <- attr(hcst, 'Variables')$common$time

#////////////////
# Method 1: use hcst time attr as obs time selector
#////////////////
  obs <- Start(dat = file.path(path.obs, files.obs),
               var = var,
               file_date = unique(format(dates, '%Y%m')),
               time = dates, # [syear, time]
               time_across = 'file_date',
               merge_across_dims = TRUE,
               split_multiselected_dims = TRUE,
               latitude = values(list(lat_min, lat_max)),
               latitude_reorder = Sort(decreasing = TRUE), 
               longitude = values(list(lon_min, lon_max)),
               longitude_reorder = circularsort,
               transform = regrid_params$obs.transform,
               transform_extra_cells = 2,
               transform_params = list(grid = regrid_params$obs.gridtype, #nc file
                                       method = regrid_params$obs.gridmethod),
               transform_vars = c('latitude', 'longitude'),
               synonims = list(latitude = c('lat','latitude'), 
                               longitude = c('lon','longitude')),
               return_vars = list(latitude = NULL, longitude = NULL,
                                  time = 'file_date'),
               retrieve = TRUE)

#////////////////
# Method 2: reshape hcst time attr's date into an array with time dim then as obs date selector
#////////////////
dim(dates)
#syear  time 
#    2    14 
dates_arr <- array(format(dates, '%Y%m'), dim = dim(dates))

  obs <- Start(dat = file.path(path.obs, files.obs),
               var = var,
               file_date = dates_arr, # [syear, time]
#               time = dates,
#               time_across = 'file_date',
#               merge_across_dims = TRUE,
               split_multiselected_dims = TRUE,
               latitude = values(list(lat_min, lat_max)),
               latitude_reorder = Sort(decreasing = TRUE),
               longitude = values(list(lon_min, lon_max)),
               longitude_reorder = circularsort,
               transform = regrid_params$obs.transform,
               transform_extra_cells = 2,
               transform_params = list(grid = regrid_params$obs.gridtype, #nc file
                                       method = regrid_params$obs.gridmethod),
               transform_vars = c('latitude', 'longitude'),
               synonims = list(latitude = c('lat','latitude'),
                               longitude = c('lon','longitude')),
               return_vars = list(latitude = NULL, longitude = NULL,
                                  time = 'file_date'),
               retrieve = TRUE)
#PROBLEMMMMMMMMMMMM
dim(attr(obs, 'Variables')$common$time)
#syear  time  time  time  time 
#    2    14     1    14     1 


#-------------------------------------------
# Step 3. Verify the consistance btwn hcst and obs
#-------------------------------------------
  # dimension
  if (any(!names(dim(obs)) %in% names(dim(hcst)))) {
    stop("hcst and obs don't share the same dimension names.")
  } else {
    match_ind <- match(names(dim(obs)), names(dim(hcst)))
    if (!all(dim(hcst)[match_ind] == dim(obs))) stop("hcst and obs don't share the same dimension length.")
  }

  # time attribute
  if (!identical(format(attr(hcst, 'Variables')$common$time, '%Y%m'),
                 format(attr(obs, 'Variables')$common$time, '%Y%m')))
    stop("hcst and obs don't share the same time.")

  # lat and lon attributes
  if (!identical(as.vector(attr(hcst, 'Variables')$common$latitude),
                 as.vector(attr(obs, 'Variables')$common$latitude)))
    stop("hcst and obs don't share the same latitude.")
  if (!identical(as.vector(attr(hcst, 'Variables')$common$longitude),
                 as.vector(attr(obs, 'Variables')$common$longitude)))
    stop("hcst and obs don't share the same longitude.")


#-------------------------------------------
# Step 4. Convert from startR_array to s2dv_array
#-------------------------------------------


